{===============================================================================


Component:    TWinZoom
Version:      1.2
Created:      01-06-2001
Author:       Aart Gerde
Copyright:     2001 EasterEgg, Aart Gerde
Status:       FREEWARE - see conditions below

Conditions:   * You can use the TWinZoom component in any application you like -
                commercial, shareware or freeware:) - without having to give me
                any credits.
              *	However, the TWinZoom component itself (or any other component
                that is a descendant of TWinZoom!) may not be sold - you got it
                for free, so give it away for free:) to as many people as you
                like. Just don't modify the source code and claim it to be your
                own or leave out any commentary in this unit when distributing it.
                If you make a component wich has TWinZoom as an ancestor, you
                should give me proper credits.
              * When you distribute it, don't just distribute the .dcu file but
                the .pas file as well, along with the original WinZoom.dcr -
                I put a lot of work in that too;).

Disclaimer:   The TWinZoom component has been created using Delphi 4.0 professional
              on a Celeron 400 Mhz PC, Windows 98. It has not been tested with
              any other version of Windows or Delphi. The TWinZoom component is
              provided "as is", without any guarantee of any kind. By using the
              TWinZoom component you declare to have read and accept this disclaimer
              and agree to use the TWinZoom component at your own risk without ever
              holding me (or anyone else who distributed the TWinZoom component
              in its original form to you) liable for any damage or loss of any
              kind that may arise out of the use of the TWinZoom component, or
              any other component that is a descendant of the TWinZoom component.

Note:         Although the TWinZoom component has been developed using Delphi 4.0,
              it seems unlikely to me that using it with an earlier version of
              Delphi will give any trouble, except for Delphi 1.0.

Description:  This component enables the form it is dropped onto to show zoom
              animation whenever the form is minimized, maximized or restored.
              It plays the sounds that are associated with those events and also
              has event handlers for them.

              Delphi (2.0 - 4.0) strangely enough doesn't add this standard
              windows behaviour to it's forms (due to the underlying application
              window). TWinZoom does. The application window shows a button
              in the taskbar and by default, the form does not. That's why the text
              on the button in the taskbar differs from the forms caption
              (e.g. Project1 - Form1), wich is also not standard behaviour in Windows.
              TWinZoom fixes this too by hiding the application window button and
              showing only the button of the form. TWinZoom enables the form to
              show a button by setting its extended window style to WS_EX_APPWINDOW.

              TWinZoom has the following events:

              - OnMinimize
              - OnMaximize
              - OnRestoreDown
              - OnRestoreUp

              The OnMinimize, OnMaximize, OnRestoreDown and OnRestoreUp event
              handlers are ment for whatever you want to happen when the form is
              respectively minimized, maximized or restored (up or down), so
              there's no need to trap the WM_SYSCOMMAND messages send to the form
              in order to do so. 

              Enjoy:)

              Aart Gerde
              e-mail: easteregg@subdimension.com

History       1.1 - Initial release;
              1.2 - TWinZoom now executes the Application.OnMinimize and
                    Application.OnRestore event handlers (if any) when the
                    form is minimized or restored;
                  - Removed a few unnecessary lines;


===============================================================================}

unit WinZoom;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, MMSystem, registry;

type
  TRestore = (rsUp, rsDown);

  TWinZoom = class(TComponent)
  private
    { Private declarations }
    FOnMinimize: TNotifyEvent;
    FOnMaximize: TNotifyEvent;
    FOnRestoreUp: TNotifyEvent;
    FOnRestoreDown: TNotifyEvent;
    FSoundMinimize: AnsiString;
    FSoundMaximize: AnsiString;
    FSoundRestoreUp: AnsiString;
    FSoundRestoreDown: AnsiString;
    FRestore: TRestore;
    FForm: TForm;
    FFormOldExStyle: Integer;
    FShowAppWindowHidden: Boolean;
    FAlreadyPresent: Boolean;
    FOldAppExceptionHandler: TExceptionEvent;
    FOldWindowProc: TWndMethod;
    procedure NewWindowProc(var Msg: TMessage);
    procedure NewAppExceptionHandler(Sender: TObject; E: Exception);
  protected
    { Protected declarations }
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
    property OnMaximize: TNotifyEvent read FOnMaximize write FOnMaximize;
    property OnRestoreUp: TNotifyEvent read FOnRestoreUp write FOnRestoreUp;
    property OnRestoreDown: TNotifyEvent read FOnRestoreDown write FOnRestoreDown;
  end;

procedure Register;

implementation

{ TWinZoom }

constructor TWinZoom.Create(AOwner: TComponent);
const
  Key = '\AppEvents\Schemes\Apps\.Default\%s\.current';
var
  ExStyle: Integer;
  Reg: TRegistry;
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then
  begin
    { Subclass the WindowProc of the underlying form }
    FForm := (AOwner as TForm);
    FOldWindowProc := FForm.WindowProc;
    FForm.WindowProc := NewWindowProc;
    { Make sure the window button of the form becomes visible in the taskbar
      by setting WS_EX_APPWINDOW as the new extended window style of the form }
    FFormOldExStyle := GetWindowLong(FForm.Handle, GWL_EXSTYLE);
    ExStyle := FFormOldExStyle or WS_EX_APPWINDOW;
    SetWindowLong(FForm.Handle, GWL_EXSTYLE, ExStyle);
    { Get info about sound events }
    Reg := nil;
    try
      Reg := TRegistry.Create;
      with Reg do
      begin
        RootKey := HKEY_CURRENT_USER;
        if OpenKey(Format(Key, ['Minimize']), False) then
          FSoundMinimize := ReadString('');
        if OpenKey(Format(Key, ['Maximize']), False) then
          FSoundMaximize := ReadString('');
        if OpenKey(Format(Key, ['RestoreUp']), False) then
          FSoundRestoreUp := ReadString('');
        if OpenKey(Format(Key, ['RestoreDown']), False) then
          FSoundRestoreDown := ReadString('');
      end;
    finally
      Reg.Free;
    end;
  end;
end;

destructor TWinZoom.Destroy;
begin
  inherited Destroy;
  if not Assigned(FForm) then
    Exit;
  if not (csDesigning in ComponentState) then
  begin
    { Restore the original WindowProc of the form }
    FForm.WindowProc:= FOldWindowProc;
    if not (csDestroying in FForm.ComponentState) then
    begin
      { Restore the original extended window style only if the form is
        not about to be destroyed. Otherwise, this would be useless }
      SetWindowLong(FForm.Handle, GWL_EXSTYLE, FFormOldExStyle);
      ShowWindow(Application.Handle, SW_SHOW);
      Application.ProcessMessages;
    end;
  end;
end;

procedure TWinZoom.NewWindowProc(var Msg: TMessage);
  procedure PlayCurrentSound(var Sound: AnsiString);
  begin
    if Sound <> '' then
      PlaySound(PChar(Sound), 0, SND_FILENAME or SND_ASYNC);
  end;
begin
  try
    case Msg.Msg of
      WM_PAINT:
        begin
          if not FShowAppWindowHidden then
          begin
            FShowAppWindowHidden := True;
            { This causes the window button of the application window to disappear
              from the taskbar when the form receives it first WM_PAINT message }
            ShowWindow(Application.Handle, SW_HIDE);
          end else
            { After that, the original WindowProc handles the WM_PAINT messages }
            FOldWindowProc(Msg);
        end;
      WM_SYSCOMMAND:
        begin
          case Msg.WParam of
            SC_MINIMIZE:
              begin
                FRestore := rsUp;
                PlayCurrentSound(FSoundMinimize);
                ShowWindow(FForm.Handle, SW_MINIMIZE);
                if Assigned(FOnMinimize) then
                  FOnMinimize(Self);
                if Assigned(Application.OnMinimize) then
                  Application.OnMinimize(Self);
              end;
            SC_RESTORE:
              begin
                { FRestore = rsUp: the form is restored from being minimized,
                  FRestore = rsDown: the form is restored from being maximized }
                case FRestore of
                  rsUp: PlayCurrentSound(FSoundRestoreUp);
                  rsDown: PlayCurrentSound(FSoundRestoreDown);
                end;
                ShowWindow(FForm.Handle, SW_RESTORE);
                case FRestore of
                  rsUp: if Assigned(FOnRestoreUp) then
                      FOnRestoreUp(Self);
                  rsDown: if Assigned(FOnRestoreDown) then
                      FOnRestoreDown(Self);
                end;
                if Assigned(Application.OnRestore) then
                  Application.OnRestore(Self);
              end;
            SC_MAXIMIZE:
              begin
                FRestore := rsDown;
                PlayCurrentSound(FSoundMaximize);
                ShowWindow(FForm.Handle, SW_MAXIMIZE);
                if Assigned(FOnMaximize) then
                  FOnMaximize(Self);
              end;
          else
            { Let all other system commands (SC_xxx), like SC_CLOSE for example,
              be handled by the original WindowProc }
            FOldWindowProc(Msg);
          end;
        end;
    else
      { Any window messages (WM_xxx) other then WM_PAINT or WM_SYSCOMMAND
        are handled by the original WindowProc }
      FOldWindowProc(Msg);
    end;
  except
    { Should anything go wrong, the original WindowProc takes care of
      handling the message }
    FOldWindowProc(Msg);
  end;
end;

procedure TWinZoom.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  { When another Winzoom component is created on the same form it will be noticed
    here. The other Winzoom component will be removed, since it is useless to
    have more than one of these on the same form - there can only be one;) }
  inherited Notification(AComponent, Operation);
  if Operation = opInsert then
    if (AComponent.ClassType = TWinZoom) and (FAlreadyPresent) then
    begin
      FOldAppExceptionHandler := Application.OnException;
      { Temporary replace the applications exception handler by
        NewAppExceptionHandler }
      Application.OnException := NewAppExceptionHandler;
      Owner.RemoveComponent(AComponent);
      { The above causes an EListError wich is handled by NewAppExceptionHandler }
    end;
  FAlreadyPresent := True;
end;

procedure TWinZoom.NewAppExceptionHandler(Sender: TObject; E: Exception);
var
  SOneWinZoomOnly: AnsiString;
  Flags: Integer;
begin
  try
    if E is EListError then
    begin
      { Caused by removing a superfluous WinZoom component }
      Flags := MB_OK or MB_ICONERROR;
      SOneWinZoomOnly := Format('%s already contains a Winzoom component.',
        [(Owner as TForm).Name]);
      MessageBox(0, PChar(SOneWinZoomOnly), 'Error', Flags);
      { Immediately restore the original Application exception handler }
      Application.OnException := FOldAppExceptionHandler;
    end
    else
      FOldAppExceptionHandler(Sender, E);
  except
    Application.OnException := FOldAppExceptionHandler;
  end;
end;

procedure Register;
begin
  RegisterComponents('Standard', [TWinZoom]);
end;

end.

